module SupportTypesTests

open System
open Expecto
open MyWebLog
open NodaTime

/// Tests for the NodaTime-wrapping module
let nodaTests = testList "Noda" [
    test "epoch succeeds" {
        Expect.equal
            (Noda.epoch.ToDateTimeUtc())
            (DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc))
            "The Unix epoch value is not correct"
    }
    test "toSecondsPrecision succeeds" {
        let testDate = Instant.FromDateTimeUtc(DateTime(1970, 1, 1, 0, 0, 0, 444, DateTimeKind.Utc))
        // testDate.
        Expect.equal
            ((Noda.toSecondsPrecision testDate).ToDateTimeUtc())
            (Noda.epoch.ToDateTimeUtc())
            "Instant value was not rounded to seconds precision"
    }
    test "fromDateTime succeeds" {
        let testDate = DateTime(1970, 1, 1, 0, 0, 0, 444, DateTimeKind.Utc)
        Expect.equal (Noda.fromDateTime testDate) Noda.epoch "fromDateTime did not truncate to seconds"
    }
]

/// Tests for the AccessLevel type
let accessLevelTests = testList "AccessLevel" [
    testList "Parse" [
        test "succeeds for \"Author\"" {
            Expect.equal Author (AccessLevel.Parse "Author") "Author not parsed correctly"
        }
        test "succeeds for \"Editor\"" {
            Expect.equal Editor (AccessLevel.Parse "Editor") "Editor not parsed correctly"
        }
        test "succeeds for \"WebLogAdmin\"" {
            Expect.equal WebLogAdmin (AccessLevel.Parse "WebLogAdmin") "WebLogAdmin not parsed correctly"
        }
        test "succeeds for \"Administrator\"" {
            Expect.equal Administrator (AccessLevel.Parse "Administrator") "Administrator not parsed correctly"
        }
        test "fails when given an unrecognized value" {
            Expect.throwsT<ArgumentException>
                (fun () -> ignore (AccessLevel.Parse "Hacker")) "Invalid value should have raised an exception"
        }
    ]
    testList "ToString" [
        test "Author succeeds" {
            Expect.equal (string Author) "Author" "Author string incorrect"
        }
        test "Editor succeeds" {
            Expect.equal (string Editor) "Editor" "Editor string incorrect"
        }
        test "WebLogAdmin succeeds" {
            Expect.equal (string WebLogAdmin) "WebLogAdmin" "WebLogAdmin string incorrect"
        }
        test "Administrator succeeds" {
            Expect.equal (string Administrator) "Administrator" "Administrator string incorrect"
        }
    ]
    testList "HasAccess" [
        test "Author has Author access" {
            Expect.isTrue (Author.HasAccess Author) "Author should have Author access"
        }
        test "Author does not have Editor access" {
            Expect.isFalse (Author.HasAccess Editor) "Author should not have Editor access"
        }
        test "Author does not have WebLogAdmin access" {
            Expect.isFalse (Author.HasAccess WebLogAdmin) "Author should not have WebLogAdmin access"
        }
        test "Author does not have Administrator access" {
            Expect.isFalse (Author.HasAccess Administrator) "Author should not have Administrator access"
        }
        test "Editor has Author access" {
            Expect.isTrue (Editor.HasAccess Author) "Editor should have Author access"
        }
        test "Editor has Editor access" {
            Expect.isTrue (Editor.HasAccess Editor) "Editor should have Editor access"
        }
        test "Editor does not have WebLogAdmin access" {
            Expect.isFalse (Editor.HasAccess WebLogAdmin) "Editor should not have WebLogAdmin access"
        }
        test "Editor does not have Administrator access" {
            Expect.isFalse (Editor.HasAccess Administrator) "Editor should not have Administrator access"
        }
        test "WebLogAdmin has Author access" {
            Expect.isTrue (WebLogAdmin.HasAccess Author) "WebLogAdmin should have Author access"
        }
        test "WebLogAdmin has Editor access" {
            Expect.isTrue (WebLogAdmin.HasAccess Editor) "WebLogAdmin should have Editor access"
        }
        test "WebLogAdmin has WebLogAdmin access" {
            Expect.isTrue (WebLogAdmin.HasAccess WebLogAdmin) "WebLogAdmin should have WebLogAdmin access"
        }
        test "WebLogAdmin does not have Administrator access" {
            Expect.isFalse (WebLogAdmin.HasAccess Administrator) "WebLogAdmin should not have Administrator access"
        }
        test "Administrator has Author access" {
            Expect.isTrue (Administrator.HasAccess Author) "Administrator should have Author access"
        }
        test "Administrator has Editor access" {
            Expect.isTrue (Administrator.HasAccess Editor) "Administrator should have Editor access"
        }
        test "Administrator has WebLogAdmin access" {
            Expect.isTrue (Administrator.HasAccess WebLogAdmin) "Administrator should have WebLogAdmin access"
        }
        test "Administrator has Administrator access" {
            Expect.isTrue (Administrator.HasAccess Administrator) "Administrator should have Administrator access"
        }
    ]
]

/// Tests for the CommentStatus type
let commentStatusTests = testList "CommentStatus" [
    testList "Parse" [
        test "succeeds for \"Approved\"" {
            Expect.equal Approved (CommentStatus.Parse "Approved") "Approved not parsed correctly"
        }
        test "succeeds for \"Pending\"" {
            Expect.equal Pending (CommentStatus.Parse "Pending") "Pending not parsed correctly"
        }
        test "succeeds for \"Spam\"" {
            Expect.equal Spam (CommentStatus.Parse "Spam") "Spam not parsed correctly"
        }
        test "fails for unrecognized value" {
            Expect.throwsT<ArgumentException>
                (fun () -> ignore (CommentStatus.Parse "Live")) "Invalid value should have raised an exception"
        }
    ]
    testList "ToString" [
        test "Approved succeeds" {
            Expect.equal (string Approved) "Approved" "Approved string incorrect"
        }
        test "Pending succeeds" {
            Expect.equal (string Pending) "Pending" "Pending string incorrect"
        }
        test "Spam succeeds" {
            Expect.equal (string Spam) "Spam" "Spam string incorrect"
        }
    ]
]

/// Tests for the ExplicitRating type
let explicitRatingTests = testList "ExplicitRating" [
    testList "Parse" [
        test "succeeds for \"yes\"" {
            Expect.equal Yes (ExplicitRating.Parse "yes") "\"yes\" not parsed correctly"
        }
        test "succeeds for \"no\"" {
            Expect.equal No (ExplicitRating.Parse "no") "\"no\" not parsed correctly"
        }
        test "succeeds for \"clean\"" {
            Expect.equal Clean (ExplicitRating.Parse "clean") "\"clean\" not parsed correctly"
        }
        test "fails for unrecognized value" {
            Expect.throwsT<ArgumentException>
                (fun () -> ignore (ExplicitRating.Parse "maybe")) "Invalid value should have raised an exception"
        }
    ]
    testList "ToString" [
        test "Yes succeeds" {
            Expect.equal (string Yes) "yes" "Yes string incorrect"
        }
        test "No succeeds" {
            Expect.equal (string No) "no" "No string incorrect"
        }
        test "Clean succeeds" {
            Expect.equal (string Clean) "clean" "Clean string incorrect"
        }
    ]
]

/// Tests for the Episode type
let episodeTests = testList "Episode" [
    testList "FormatDuration" [
        test "succeeds when no duration is specified" {
            Expect.isNone (Episode.Empty.FormatDuration()) "A missing duration should have returned None"
        }
        test "succeeds when duration is specified" {
            Expect.equal
                ({ Episode.Empty with
                    Duration = Some (Duration.FromMinutes 3L + Duration.FromSeconds 13L) }.FormatDuration())
                (Some "0:03:13")
                "Duration not formatted correctly"
        }
        test "succeeds when duration is > 10 hours" {
            Expect.equal
                ({ Episode.Empty with Duration = Some (Duration.FromHours 11) }.FormatDuration())
                (Some "11:00:00")
                "Duration not formatted correctly"
        }
    ]
]

/// Unit tests for the MarkupText type
let markupTextTests = testList "MarkupText" [
    testList "Parse" [
        test "succeeds with HTML content" {
            let txt = MarkupText.Parse "HTML: <p>howdy</p>"
            match txt with
            | Html it when it = "<p>howdy</p>" -> ()
            | _ -> Expect.isTrue false $"Unexpected parse result for HTML: %A{txt}"
        }
        test "succeeds with Markdown content" {
            let txt = MarkupText.Parse "Markdown: # A Title"
            match txt with
            | Markdown it when it = "# A Title" -> ()
            | _ -> Expect.isTrue false $"Unexpected parse result for Markdown: %A{txt}"
        }
        test "fails with unexpected content" {
            Expect.throwsT<ArgumentException>
                (fun () -> ignore (MarkupText.Parse "LaTEX: nope")) "Invalid value should have raised an exception"
        }
    ]
    testList "SourceType" [
        test "succeeds for HTML" {
            Expect.equal (MarkupText.Parse "HTML: something").SourceType "HTML" "HTML source type incorrect"
        }
        test "succeeds for Markdown" {
            Expect.equal (MarkupText.Parse "Markdown: blah").SourceType "Markdown" "Markdown source type incorrect"
        }
    ]
    testList "Text" [
        test "succeeds for HTML" {
            Expect.equal (MarkupText.Parse "HTML: test").Text "test" "HTML text incorrect"
        }
        test "succeeds for Markdown" {
            Expect.equal (MarkupText.Parse "Markdown: test!").Text "test!" "Markdown text incorrect"
        }
    ]
    testList "ToString" [
        test "succeeds for HTML" {
            Expect.equal
                (string (MarkupText.Parse "HTML: <h1>HTML</h1>")) "HTML: <h1>HTML</h1>" "HTML string value incorrect"
        }
        test "succeeds for Markdown" {
            Expect.equal
                (string (MarkupText.Parse "Markdown: # Some Content"))
                "Markdown: # Some Content"
                "Markdown string value incorrect"
        }
    ]
    testList "AsHtml" [
        test "succeeds for HTML" {
            Expect.equal
                ((MarkupText.Parse "HTML: <h1>The Heading</h1>").AsHtml()) "<h1>The Heading</h1>" "HTML value incorrect"
        }
        test "succeeds for Markdown" {
            Expect.equal
                ((MarkupText.Parse "Markdown: *emphasis*").AsHtml())
                "<p><em>emphasis</em></p>\n"
                "Markdown HTML value incorrect"
        }
    ]
]

/// Unit tests for the OpenGraphAudio type
let openGraphAudioTests = testList "OpenGraphAudio" [
    testList "Properties" [
        test "succeeds with minimum required" {
            let props = Array.ofSeq { OpenGraphAudio.Empty with Url = "http://test.this" }.Properties
            Expect.hasLength props 1 "There should be one property"
            Expect.equal props[0] ("og:audio", "http://test.this") "The URL was not written correctly"
        }
        test "succeeds with secure URL" {
            let props = Array.ofSeq { OpenGraphAudio.Empty with Url = "https://test.this" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[0] ("og:audio", "https://test.this") "The URL was not written correctly"
            Expect.equal
                props[1] ("og:audio:secure_url", "https://test.this") "The Secure URL was not written correctly"
        }
        test "succeeds with all properties filled" {
            let props =
                { Url       = "http://test.this"
                  SecureUrl = Some "https://test.other"
                  Type      = Some "audio/mpeg" }.Properties
                |> Array.ofSeq
            Expect.hasLength props 3 "There should be three properties"
            Expect.equal props[0] ("og:audio", "http://test.this") "The URL was not written correctly"
            Expect.equal
                props[1] ("og:audio:secure_url", "https://test.other") "The Secure URL was not written correctly"
            Expect.equal props[2] ("og:audio:type", "audio/mpeg") "The MIME type was not written correctly"
        }
        test "succeeds when deriving AAC" {
            let props = Array.ofSeq { OpenGraphAudio.Empty with Url = "/this/cool.file.aac" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:audio:type", "audio/aac") "The MIME type for AAC was not derived correctly"
        }
        test "succeeds when deriving MP3" {
            let props = Array.ofSeq { OpenGraphAudio.Empty with Url = "/an.other/song.mp3" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:audio:type", "audio/mpeg") "The MIME type for MP3 was not derived correctly"
        }
        test "succeeds when deriving OGA" {
            let props = Array.ofSeq { OpenGraphAudio.Empty with Url = "/talks/speex.oga" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:audio:type", "audio/ogg") "The MIME type for OGA was not derived correctly"
        }
        test "succeeds when deriving WAV" {
            let props = Array.ofSeq { OpenGraphAudio.Empty with Url = "/some/old.school.wav" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:audio:type", "audio/wav") "The MIME type for WAV was not derived correctly"
        }
        test "succeeds when deriving WEBA" {
            let props = Array.ofSeq { OpenGraphAudio.Empty with Url = "/new/format/file.weba" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:audio:type", "audio/webm") "The MIME type for WEBA was not derived correctly"
        }
        test "succeeds when type cannot be derived" {
            let props = Array.ofSeq { OpenGraphAudio.Empty with Url = "/profile.jpg" }.Properties
            Expect.hasLength props 1 "There should be one property (only URL; no type derived)"
        }
    ]
]

/// Tests for the OpenGraphImage type
let openGraphImageTests = testList "OpenGraphImage" [
    testList "Properties" [
        test "succeeds with minimum required" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "http://test.url" }.Properties
            Expect.hasLength props 1 "There should be one property"
            Expect.equal props[0] ("og:image", "http://test.url") "The URL was not written correctly"
        }
        test "succeeds with secure URL" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "https://secure.url" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[0] ("og:image", "https://secure.url") "The URL was not written correctly"
            Expect.equal
                props[1] ("og:image:secure_url", "https://secure.url") "The Secure URL was not written correctly"
        }
        test "succeeds with all properties filled" {
            let props =
                { Url       = "http://test.this"
                  SecureUrl = Some "https://test.other"
                  Type      = Some "image/jpeg"
                  Width     = Some 400
                  Height    = Some 600
                  Alt       = Some "This ought to be good" }.Properties
                |> Array.ofSeq
            Expect.hasLength props 6 "There should be six properties"
            Expect.equal props[0] ("og:image", "http://test.this") "The URL was not written correctly"
            Expect.equal
                props[1] ("og:image:secure_url", "https://test.other") "The Secure URL was not written correctly"
            Expect.equal props[2] ("og:image:type", "image/jpeg") "The MIME type was not written correctly"
            Expect.equal props[3] ("og:image:width", "400") "The width was not written correctly"
            Expect.equal props[4] ("og:image:height", "600") "The height was not written correctly"
            Expect.equal props[5] ("og:image:alt", "This ought to be good") "The alt text was not written correctly"
        }
        test "succeeds when deriving BMP" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "/old/windows.bmp" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:image:type", "image/bmp") "The MIME type for BMP was not derived correctly"
        }
        test "succeeds when deriving GIF" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "/its.a.soft.g.gif" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:image:type", "image/gif") "The MIME type for GIF was not derived correctly"
        }
        test "succeeds when deriving ICO" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "/favicon.ico" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal
                props[1] ("og:image:type", "image/vnd.microsoft.icon") "The MIME type for ICO was not derived correctly"
        }
        test "succeeds when deriving JPEG" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "/big/name/photo.jpeg" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:image:type", "image/jpeg") "The MIME type for JPEG was not derived correctly"
        }
        test "succeeds when deriving PNG" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "/some/nice/graphic.png" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:image:type", "image/png") "The MIME type for PNG was not derived correctly"
        }
        test "succeeds when deriving SVG" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "/fancy-new-vector.svg" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:image:type", "image/svg+xml") "The MIME type for SVG was not derived correctly"
        }
        test "succeeds when deriving TIF" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "/tagged/file.tif" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:image:type", "image/tiff") "The MIME type for TIF was not derived correctly"
        }
        test "succeeds when deriving TIFF" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "/tagged/file.two.tiff" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:image:type", "image/tiff") "The MIME type for TIFF was not derived correctly"
        }
        test "succeeds when deriving WEBP" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "/modern/photo.webp" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:image:type", "image/webp") "The MIME type for WEBP was not derived correctly"
        }
        test "succeeds when type cannot be derived" {
            let props = Array.ofSeq { OpenGraphImage.Empty with Url = "/intro.mp3" }.Properties
            Expect.hasLength props 1 "There should be one property (only URL; no type derived)"
        }
    ]
]

/// Unit tests for the OpenGraphVideo type
let openGraphVideoTests = testList "OpenGraphVideo" [
    testList "Properties" [
        test "succeeds with minimum required" {
            let props = Array.ofSeq { OpenGraphVideo.Empty with Url = "http://url.test" }.Properties
            Expect.hasLength props 1 "There should be one property"
            Expect.equal props[0] ("og:video", "http://url.test") "The URL was not written correctly"
        }
        test "succeeds with secure URL" {
            let props = Array.ofSeq { OpenGraphVideo.Empty with Url = "https://url.secure" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[0] ("og:video", "https://url.secure") "The URL was not written correctly"
            Expect.equal
                props[1] ("og:video:secure_url", "https://url.secure") "The Secure URL was not written correctly"
        }
        test "succeeds with all properties filled" {
            let props =
                { Url       = "http://test.this"
                  SecureUrl = Some "https://test.other"
                  Type      = Some "video/mpeg"
                  Width     = Some 1200
                  Height    = Some 900 }.Properties
                |> Array.ofSeq
            Expect.hasLength props 5 "There should be five properties"
            Expect.equal props[0] ("og:video", "http://test.this") "The URL was not written correctly"
            Expect.equal
                props[1] ("og:video:secure_url", "https://test.other") "The Secure URL was not written correctly"
            Expect.equal props[2] ("og:video:type", "video/mpeg") "The MIME type was not written correctly"
            Expect.equal props[3] ("og:video:width", "1200") "The width was not written correctly"
            Expect.equal props[4] ("og:video:height", "900") "The height was not written correctly"
        }
        test "succeeds when deriving AVI" {
            let props = Array.ofSeq { OpenGraphVideo.Empty with Url = "/my.video.avi" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:video:type", "video/x-msvideo") "The MIME type for AVI was not derived correctly"
        }
        test "succeeds when deriving MP4" {
            let props = Array.ofSeq { OpenGraphVideo.Empty with Url = "/chapters/1/01.mp4" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:video:type", "video/mp4") "The MIME type for MP4 was not derived correctly"
        }
        test "succeeds when deriving MPEG" {
            let props = Array.ofSeq { OpenGraphVideo.Empty with Url = "/viral/video.mpeg" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:video:type", "video/mpeg") "The MIME type for MPEG was not derived correctly"
        }
        test "succeeds when deriving OGV" {
            let props = Array.ofSeq { OpenGraphVideo.Empty with Url = "/open/video/example.ogv" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:video:type", "video/ogg") "The MIME type for OGV was not derived correctly"
        }
        test "succeeds when deriving WEBM" {
            let props = Array.ofSeq { OpenGraphVideo.Empty with Url = "/images/hero.webm" }.Properties
            Expect.hasLength props 2 "There should be two properties"
            Expect.equal props[1] ("og:video:type", "video/webm") "The MIME type for WEBM was not derived correctly"
        }
        test "succeeds when type cannot be derived" {
            let props = Array.ofSeq { OpenGraphVideo.Empty with Url = "/favicon.ico" }.Properties
            Expect.hasLength props 1 "There should be one property (only URL; no type derived)"
        }
    ]
]

/// Unit tests for the OpenGraphType type
let openGraphTypeTests = testList "OpenGraphType" [
    testList "Parse" [
        test "succeeds for \"article\"" {
            Expect.equal (OpenGraphType.Parse "article") Article "\"article\" not parsed correctly"
        }
        test "succeeds for \"book\"" {
            Expect.equal (OpenGraphType.Parse "book") Book "\"book\" not parsed correctly"
        }
        test "succeeds for \"music.album\"" {
            Expect.equal (OpenGraphType.Parse "music.album") MusicAlbum "\"music.album\" not parsed correctly"
        }
        test "succeeds for \"music.playlist\"" {
            Expect.equal (OpenGraphType.Parse "music.playlist") MusicPlaylist "\"music.playlist\" not parsed correctly"
        }
        test "succeeds for \"music.radio_station\"" {
            Expect.equal
                (OpenGraphType.Parse "music.radio_station")
                MusicRadioStation
                "\"music.radio_station\" not parsed correctly"
        }
        test "succeeds for \"music.song\"" {
            Expect.equal (OpenGraphType.Parse "music.song") MusicSong "\"music.song\" not parsed correctly"
        }
        test "succeeds for \"payment.link\"" {
            Expect.equal (OpenGraphType.Parse "payment.link") PaymentLink "\"payment.link\" not parsed correctly"
        }
        test "succeeds for \"profile\"" {
            Expect.equal (OpenGraphType.Parse "profile") Profile "\"profile\" not parsed correctly"
        }
        test "succeeds for \"video.episode\"" {
            Expect.equal (OpenGraphType.Parse "video.episode") VideoEpisode "\"video.episode\" not parsed correctly"
        }
        test "succeeds for \"video.movie\"" {
            Expect.equal (OpenGraphType.Parse "video.movie") VideoMovie "\"video.movie\" not parsed correctly"
        }
        test "succeeds for \"video.other\"" {
            Expect.equal (OpenGraphType.Parse "video.other") VideoOther "\"video.other\" not parsed correctly"
        }
        test "succeeds for \"video.tv_show\"" {
            Expect.equal (OpenGraphType.Parse "video.tv_show") VideoTvShow "\"video.tv_show\" not parsed correctly"
        }
        test "succeeds for \"website\"" {
            Expect.equal (OpenGraphType.Parse "website") Website "\"website\" not parsed correctly"
        }
        test "fails for invalid type" {
            Expect.throwsT<ArgumentException>
                (fun () -> ignore (OpenGraphType.Parse "anthology")) "Invalid value should have raised an exception"
        }
    ]
    testList "ToString" [
        test "succeeds for Article" {
            Expect.equal (string Article) "article" "Article string incorrect"
        }
        test "succeeds for Book" {
            Expect.equal (string Book) "book" "Book string incorrect"
        }
        test "succeeds for MusicAlbum" {
            Expect.equal (string MusicAlbum) "music.album" "MusicAlbum string incorrect"
        }
        test "succeeds for MusicPlaylist" {
            Expect.equal (string MusicPlaylist) "music.playlist" "MusicPlaylist string incorrect"
        }
        test "succeeds for MusicRadioStation" {
            Expect.equal (string MusicRadioStation) "music.radio_station" "MusicRadioStation string incorrect"
        }
        test "succeeds for MusicSong" {
            Expect.equal (string MusicSong) "music.song" "MusicSong string incorrect"
        }
        test "succeeds for PaymentLink" {
            Expect.equal (string PaymentLink) "payment.link" "PaymentLink string incorrect"
        }
        test "succeeds for Profile" {
            Expect.equal (string Profile) "profile" "Profile string incorrect"
        }
        test "succeeds for VideoEpisode" {
            Expect.equal (string VideoEpisode) "video.episode" "VideoEpisode string incorrect"
        }
        test "succeeds for VideoMovie" {
            Expect.equal (string VideoMovie) "video.movie" "VideoMovie string incorrect"
        }
        test "succeeds for VideoOther" {
            Expect.equal (string VideoOther) "video.other" "VideoOther string incorrect"
        }
        test "succeeds for VideoTvShow" {
            Expect.equal (string VideoTvShow) "video.tv_show" "VideoTvShow string incorrect"
        }
        test "succeeds for Website" {
            Expect.equal (string Website) "website" "Website string incorrect"
        }
    ]
]

/// Unit tests for the OpenGraphProperties type
let openGraphPropertiesTests = testList "OpenGraphProperties" [
    testList "Properties" [
        test "succeeds with minimal values" {
            let props =
                { OpenGraphProperties.Empty with
                    Image = { OpenGraphImage.Empty with Url = "http://this.aint.nothing" } }.Properties
                |> Array.ofSeq
            Expect.hasLength props 2 "There should have been two properties"
            Expect.equal props[0] ("og:type", "article") "Type not written correctly"
            Expect.equal props[1] ("og:image", "http://this.aint.nothing") "Image URL not written correctly"
        }
        test "succeeds with all values" {
            let props =
                { Type            = Book
                  Image           = { OpenGraphImage.Empty with Url = "http://this.image.file" }
                  Audio           = Some { OpenGraphAudio.Empty with Url = "http://this.audio.file" }
                  Description     = Some "This is a unit test"
                  Determiner      = Some "a"
                  Locale          = Some "en_US"
                  LocaleAlternate = Some [ "en_UK"; "es_MX" ]
                  Video           = Some { OpenGraphVideo.Empty with Url = "http://this.video.file" }
                  Other           = Some [ { Name = "book.publisher"; Value = "Yep" } ] }.Properties
                |> Array.ofSeq
            Expect.hasLength props 10 "There should have been ten properties"
            Expect.equal props[0] ("og:type", "book") "Type not written correctly"
            Expect.equal props[1] ("og:image", "http://this.image.file") "Image URL not written correctly"
            Expect.equal props[2] ("og:description", "This is a unit test") "Description not written correctly"
            Expect.equal props[3] ("og:determiner", "a") "Determiner not written correctly"
            Expect.equal props[4] ("og:locale", "en_US") "Locale not written correctly"
            Expect.equal props[5] ("og:locale:alternate", "en_UK") "1st Alternate Locale not written correctly"
            Expect.equal props[6] ("og:locale:alternate", "es_MX") "2nd Alternate Locale not written correctly"
            Expect.equal props[7] ("og:audio", "http://this.audio.file") "Audio URL not written correctly"
            Expect.equal props[8] ("og:video", "http://this.video.file") "Video URL not written correctly"
            Expect.equal props[9] ("book.publisher", "Yep") "Other property not written correctly"
        }
    ]
]

/// Unit tests for the PodcastMedium type
let podcastMediumTests = testList "PodcastMedium" [
    testList "Parse" [
        test "succeeds for \"podcast\"" {
            Expect.equal (PodcastMedium.Parse "podcast") Podcast "\"podcast\" not parsed correctly"
        }
        test "succeeds for \"music\"" {
            Expect.equal (PodcastMedium.Parse "music") Music "\"music\" not parsed correctly"
        }
        test "succeeds for \"video\"" {
            Expect.equal (PodcastMedium.Parse "video") Video "\"video\" not parsed correctly"
        }
        test "succeeds for \"film\"" {
            Expect.equal (PodcastMedium.Parse "film") Film "\"film\" not parsed correctly"
        }
        test "succeeds for \"audiobook\"" {
            Expect.equal (PodcastMedium.Parse "audiobook") Audiobook "\"audiobook\" not parsed correctly"
        }
        test "succeeds for \"newsletter\"" {
            Expect.equal (PodcastMedium.Parse "newsletter") Newsletter "\"newsletter\" not parsed correctly"
        }
        test "succeeds for \"blog\"" {
            Expect.equal (PodcastMedium.Parse "blog") Blog "\"blog\" not parsed correctly"
        }
        test "fails for invalid type" {
            Expect.throwsT<ArgumentException>
                (fun () -> ignore (PodcastMedium.Parse "laser")) "Invalid value should have raised an exception"
        }
    ]
    testList "ToString" [
        test "succeeds for Podcast" {
            Expect.equal (string Podcast) "podcast" "Podcast string incorrect"
        }
        test "succeeds for Music" {
            Expect.equal (string Music) "music" "Music string incorrect"
        }
        test "succeeds for Video" {
            Expect.equal (string Video) "video" "Video string incorrect"
        }
        test "succeeds for Film" {
            Expect.equal (string Film) "film" "Film string incorrect"
        }
        test "succeeds for Audiobook" {
            Expect.equal (string Audiobook) "audiobook" "Audiobook string incorrect"
        }
        test "succeeds for Newsletter" {
            Expect.equal (string Newsletter) "newsletter" "Newsletter string incorrect"
        }
        test "succeeds for Blog" {
            Expect.equal (string Blog) "blog" "Blog string incorrect"
        }
    ]
]

/// Unit tests for the PostStatus type
let postStatusTests = testList "PostStatus" [
    testList "Parse" [
        test "succeeds for \"Draft\"" {
            Expect.equal (PostStatus.Parse "Draft") Draft "\"Draft\" not parsed correctly"
        }
        test "succeeds for \"Published\"" {
            Expect.equal (PostStatus.Parse "Published") Published "\"Published\" not parsed correctly"
        }
        test "fails for unrecognized value" {
            Expect.throwsT<ArgumentException>
                (fun () -> ignore (PostStatus.Parse "Rescinded")) "Invalid value should have raised an exception"
        }
    ]
]

/// Unit tests for the CustomFeedSource type
let customFeedSourceTests = testList "CustomFeedSource" [
    testList "Parse" [
        test "succeeds for category feeds" {
            Expect.equal
                (CustomFeedSource.Parse "category:abc123")
                (Category (CategoryId "abc123"))
                "Category feed not parsed correctly"
        }
        test "succeeds for tag feeds" {
            Expect.equal (CustomFeedSource.Parse "tag:turtles") (Tag "turtles") "Tag feed not parsed correctly"
        }
        test "fails for unknown type" {
            Expect.throwsT<ArgumentException>
                (fun () -> ignore (CustomFeedSource.Parse "nasa:sat1")) "Invalid value should have raised an exception"
        }
    ]
    testList "ToString" [
        test "succeeds for category feed" {
            Expect.equal
                (string (CustomFeedSource.Parse "category:fish")) "category:fish" "Category feed string incorrect"
        }
        test "succeeds for tag feed" {
            Expect.equal (string (CustomFeedSource.Parse "tag:rocks")) "tag:rocks" "Tag feed string incorrect"
        }
    ]
]

/// Unit tests for the ThemeAssetId type
let themeAssetIdTests = testList "ThemeAssetId" [
    testList "Parse" [
        test "succeeds with expected values" {
            Expect.equal
                (ThemeAssetId.Parse "test-theme/the-asset")
                (ThemeAssetId ((ThemeId "test-theme"), "the-asset"))
                "Theme asset ID not parsed correctly"
        }
        test "fails if no slash is present" {
            Expect.throwsT<ArgumentException>
                (fun () -> ignore (ThemeAssetId.Parse "my-theme-asset")) "Invalid value should have raised an exception"
        }
    ]
    test "ToString succeeds" {
        Expect.equal
            (string (ThemeAssetId ((ThemeId "howdy"), "pardner"))) "howdy/pardner" "Theme asset ID string incorrect"
    }
]

/// Unit tests for the UploadDestination type
let uploadDestinationTests = testList "UploadDestination" [
    testList "Parse" [
        test "succeeds for \"Database\"" {
            Expect.equal (UploadDestination.Parse "Database") Database "\"Database\" not parsed correctly"
        }
        test "succeeds for \"Disk\"" {
            Expect.equal (UploadDestination.Parse "Disk") Disk "\"Disk\" not parsed correctly"
        }
        test "fails for unrecognized value" {
            Expect.throwsT<ArgumentException>
                (fun () -> ignore (UploadDestination.Parse "Azure")) "Invalid value should have raised an exception"
        }
    ]
    testList "ToString" [
        test "succeeds for Database" {
            Expect.equal (string Database) "Database" "Database string incorrect"
        }
        test "succeeds for Disk" {
            Expect.equal (string Disk) "Disk" "Disk string incorrect"
        }
    ]
]

/// All tests for the Domain.SupportTypes file
let all = testList "SupportTypes" [
    nodaTests
    accessLevelTests
    commentStatusTests
    explicitRatingTests
    episodeTests
    markupTextTests
    openGraphAudioTests
    openGraphImageTests
    openGraphVideoTests
    openGraphTypeTests
    openGraphPropertiesTests
    podcastMediumTests
    postStatusTests
    customFeedSourceTests
    themeAssetIdTests
    uploadDestinationTests
]
